==> lein-depgraph-0.1.1.jar: leiningen/depgraph.clj <==
(ns leiningen.depgraph
  "Generate a namespace dependency graph as an svg file"
  (:use [clojure.java.shell :only [sh]]))


(defn ffile [file]
  (if (string? file)
    (java.io.File. file)
    file))

(defn read-ns [file]
  (with-open [f (-> file ffile java.io.FileReader.
                java.io.PushbackReader.)]
    (binding [*in* f]
      (let [x (read)]
        (if (= (first x) 'ns)
          x
          nil)))))

(defn parse-reference-entry [ref]
  (if-not (sequential? ref)
    [ref]
    (let [ext (loop [[c & r] (rest ref), ns-ext ()]
                (cond (or (keyword? c) (nil? c)) ns-ext
                      :else (recur r (conj ns-ext (if (sequential? c)
                                                    (first c) c)))))]
      (if (empty? ext)
        [(first ref)]
        (map #(symbol (str (first ref) \. %)) ext)))))

(defn get-required [ns-form]
  (->> ns-form
       (filter #(and (coll? %) (= :require (first %))))
       (mapcat rest)
       (mapcat parse-reference-entry)
       set))

(defn get-used [ns-form]
  (->> ns-form
      (filter #(and (coll? %) (= :use (first %))))
      (mapcat rest)
      (mapcat parse-reference-entry)
      set))

(defn get-import [ns-form]
  (->> ns-form
       (filter #(and (coll? %) (= :import (first %))))
       (mapcat rest)
       (mapcat parse-reference-entry)
       set))

(defn parse [ns-form]
  {:name (second ns-form)
   :clojure-depends (concat (get-required ns-form)
                            (get-used ns-form))
   :java-depends (get-import ns-form)})

(defn all-clojure-files [root]
  (let [root (ffile root)]
    (if (.isDirectory root)
      (mapcat all-clojure-files (.listFiles root
                                            (proxy [java.io.FilenameFilter] []
                                              (accept [dir name]
                                                      (or (.endsWith name ".clj")
                                                          (.isDirectory (java.io.File. dir name)))))))
      [root])))



(defn parse-directory [dir]
  (reduce #(assoc % (:name %2) (dissoc %2 :name)) {}
          (map parse (remove nil? (map read-ns (all-clojure-files dir))))))

(defn restructure [files]
(reduce
  (fn [map- dep]
    (-> map- (update-in [:java] #(into % (:java-depends (second dep))))
      (update-in [:clojure] #(conj (into % (:clojure-depends (second dep)))
                                   (first dep)))
      (update-in [:edges]
                 #(into %
                        (map (partial vector (first dep))
                             (concat (:java-depends (second dep))
                                     (:clojure-depends (second dep))))))))
  {:java #{} :clojure #{} :edges #{}}
  files))

(defn dot [x]
  (format "digraph simple_hierarchy {\n graph [rankdir = \"LR\"];\n %s subgraph cluster_clojure {\nlabel=\"clojure\";\ncolor=blue;\n%s} subgraph cluster_java {\nlabel=\"java\";\ncolor=red;\n%s}}" (:edges x) (:clojure x) (:java x)))
  
(defn safe-name [string]
  (str (.replaceAll (str string) "(\\.|-|\\$)" "_")))

(defn safe-name-and-label [structure tag]
  (update-in structure [tag]
             #(reduce str (map
                (fn [x]
                  (str (safe-name x) "[label=\"" x "\"];\n")) %))))

(defn edges [structure]
  (update-in structure [:edges]
             (fn [x]
               (reduce str (map #(format "%s->%s;\n" (safe-name (first %)) (safe-name (second %))) x)))))

(defn depgraph
  "Generate a namespace dependency graph as svg file"
  [project]
  (let [source-path (:source-path project "src")
        dotfile (str (:name project) ".dot")
        svgfile (str (:name project) ".svg")]
    (-> source-path
        parse-directory
        ((partial remove (comp nil? first)))
        restructure
        (safe-name-and-label :java)
        (safe-name-and-label :clojure)
        edges
        dot
        ((partial spit dotfile)))
    (sh "dot" "-Tsvg" (str "-o" svgfile ) dotfile) ))

;; (depgraph {:name "lein-droid", :source-path "/home/unlogic/clojure/android/lein-droid/src"})

==> lein-depgraph-0.1.2-20120710.194547-1.jar: leiningen/depgraph.clj <==
(ns leiningen.depgraph
  "Generate a namespace dependency graph as an svg file"
  (:use [clojure.java.shell :only [sh]]))

(defn ffile [file]
  (if (string? file)
    (java.io.File. file)
    file))

(defn read-ns [file]
  (with-open [f (-> file ffile java.io.FileReader.
                java.io.PushbackReader.)]
    (binding [*in* f]
      (let [x (read)]
        (if (= (first x) 'ns)
          x
          nil)))))

(defn parse-reference-entry [ref]
  (if-not (sequential? ref)
    [ref]
    (let [ext (loop [[c & r] (rest ref), ns-ext ()]
                (cond (or (keyword? c) (nil? c)) ns-ext
                      :else (recur r (conj ns-ext (if (sequential? c)
                                                    (first c) c)))))]
      (if (empty? ext)
        [(first ref)]
        (map #(symbol (str (first ref) \. %)) ext)))))

(defn get-required [ns-form]
  (->> ns-form
       (filter #(and (coll? %) (= :require (first %))))
       (mapcat rest)
       (mapcat parse-reference-entry)
       set))

(defn get-used [ns-form]
  (->> ns-form
      (filter #(and (coll? %) (= :use (first %))))
      (mapcat rest)
      (mapcat parse-reference-entry)
      set))

(defn get-import [ns-form]
  (->> ns-form
       (filter #(and (coll? %) (= :import (first %))))
       (mapcat rest)
       (mapcat parse-reference-entry)
       set))

(defn parse [ns-form]
  {:name (second ns-form)
   :clojure-depends (concat (get-required ns-form)
                            (get-used ns-form))
   :java-depends (get-import ns-form)})

(defn all-clojure-files [root]
  (let [root (ffile root)]
    (if (.isDirectory root)
      (mapcat all-clojure-files (.listFiles root
                                            (proxy [java.io.FilenameFilter] []
                                              (accept [dir name]
                                                      (or (.endsWith name ".clj")
                                                          (.isDirectory (java.io.File. dir name)))))))
      [root])))



(defn parse-directory [dir]
  (reduce #(assoc % (:name %2) (dissoc %2 :name)) {}
          (map parse (remove nil? (map read-ns (all-clojure-files dir))))))

(defn restructure [files]
(reduce
  (fn [map- dep]
    (-> map- (update-in [:java] #(into % (:java-depends (second dep))))
      (update-in [:clojure] #(conj (into % (:clojure-depends (second dep)))
                                   (first dep)))
      (update-in [:edges]
                 #(into %
                        (map (partial vector (first dep))
                             (concat (:java-depends (second dep))
                                     (:clojure-depends (second dep))))))))
  {:java #{} :clojure #{} :edges #{}}
  files))

(defn dot [x]
  (format "digraph simple_hierarchy {\n graph [rankdir = \"LR\"];\n %s subgraph cluster_clojure {\nlabel=\"clojure\";\ncolor=blue;\n%s} subgraph cluster_java {\nlabel=\"java\";\ncolor=red;\n%s}}" (:edges x) (:clojure x) (:java x)))

(defn together-dot [x]
  (format "digraph simple_hierarchy {\n graph [rankdir = \"LR\"];\n %s\n%s\n%s}" (:edges x) (:clojure x) (:java x)))

(defn safe-name [string]
  (str (.replaceAll (str string) "(\\.|-|\\$)" "_")))

(defn safe-name-and-label [structure tag shape]
  (update-in structure [tag]
             #(reduce str (map
                (fn [x]
                  (str (safe-name x) "[label=\"" x "\",shape=\"" shape "\"];\n")) %))))

(defn edges [structure]
  (update-in structure [:edges]
             (fn [x]
               (reduce str (map #(format "%s->%s;\n" (safe-name (first %)) (safe-name (second %))) x)))))

(defn depgraph
  "Generate a namespace dependency graph as svg file"
  [project & [arg]]
  (let [source-path (:source-path project "src")
        dotfile (str (:name project) ".dot")
        svgfile (str (:name project) ".svg")]
    (-> source-path
        parse-directory
        ((partial remove (comp nil? first)))
        restructure
        (safe-name-and-label :java (if (= arg "t") "hexagon" "ellipse"))
        (safe-name-and-label :clojure "ellipse")
        edges
        ((fn [x] (if (= arg "t") (together-dot x) (dot x))))
        ((partial spit dotfile)))
    (sh "dot" "-Tsvg" (str "-o" svgfile ) dotfile) ))

;; (depgraph {:name "lein-droid", :source-path "/home/unlogic/clojure/android/lein-droid/src"})
